searchData={"items":[{"type":"module","title":"Tiny","doc":"This module contains the entirety of the Tiny project. There are interface\nfunctions for encoding and decoding JSON, and little else.\n\nBoth encoding and decoding are done per the JSON specification with direct\nmapping between JSON types and Elixir types:\n\n --------------------------------\n| JSON Type |     Elixir Type    |\n|:---------:|:------------------:|\n|   Array   |        List        |\n|  Boolean  |  `true` or `false` |\n|   Object  |        Map         |\n|   Number  |  Integer or Float  |\n|   String  |       Binary       |\n|   `null`  |       `nil`        |\n --------------------------------\n\nNumbers are parsed into Integer types when possible, falling back to Floats\nwhen required. This is due to functions which operate differently based on\nthe numeric typing.","ref":"Tiny.html"},{"type":"function","title":"Tiny.decode/2","doc":"Safely decodes a JSON input binary to an Elixir term.\n\nParsing errors are caught and an `:error` Tuple is returned. Any successful\nvalues are wrapped in an `:ok` Tuple.","ref":"Tiny.html#decode/2"},{"type":"function","title":"Tiny.decode!/2","doc":"Decodes a JSON inout binary to an Elixir term.\n\nRather than return Tuples, this function will raise `ArgumentError` on invalid\ninputs. No information is provided beyond the error, so log context as needed.","ref":"Tiny.html#decode!/2"},{"type":"function","title":"Tiny.encode/2","doc":"Safely encodes a JSON compatible value to iodata or a binary.\n\nParsing errors are caught and an `:error` Tuple is returned. Any successful\nvalues are wrapped in an `:ok` Tuple.","ref":"Tiny.html#encode/2"},{"type":"function","title":"Tiny.encode!/2","doc":"Encodes a JSON compatible value to iodata or a binary.\n\nRather than return Tuples, this function will raise `ArgumentError` on invalid\ninputs. No information is provided beyond the error, so log context as needed.","ref":"Tiny.html#encode!/2"},{"type":"function","title":"Tiny.encode_to_iodata/2","doc":"Safely encodes a value to JSON as iodata.\n\nSame as passing `[ iodata: true ]` to `encode/2`.","ref":"Tiny.html#encode_to_iodata/2"},{"type":"function","title":"Tiny.encode_to_iodata!/2","doc":"Encodes a value to JSON as iodata.\n\nSame as passing `[ iodata: true ]` to `encode!/2`.","ref":"Tiny.html#encode_to_iodata!/2"},{"type":"type","title":"Tiny.json/0","doc":"","ref":"Tiny.html#t:json/0"},{"type":"extras","title":"Tiny","doc":"# Tiny\n[![Build Status](https://img.shields.io/github/actions/workflow/status/whitfin/tiny/ci.yml?branch=main)](https://github.com/whitfin/tiny/actions) [![Hex.pm Version](https://img.shields.io/hexpm/v/tiny.svg)](https://hex.pm/packages/tiny) [![Documentation](https://img.shields.io/badge/docs-latest-blue.svg)](https://hexdocs.pm/tiny/)\n\nTiny is a JSON parser for Elixir written using efficient function head matches where possible.\n\n_Update 2025: This project is quite redundant due to the inclusion of the [json](https://www.erlang.org/doc/apps/stdlib/json.html) module inside OTP. I have left this repository here primarily as an example, but also because it was fun to work on :)._","ref":"readme.html"},{"type":"extras","title":"Benefits - Tiny","doc":"This library was created due to my need for a JSON parser I could easily bundle into archive builds - thus it's a single file, and very small. If you are looking for the most efficient parser (i.e. for a web service), you should probably look elsewhere -- [Jason](https://github.com/michalmuskala/jason) and [Poison](https://github.com/devinus/poison) are good options.\n\nWhy you should use Tiny:\n\n- Allows only binary keys on decode, by design\n- Allows both binary and atom keys on encode, by design\n- Allows encoding to either binary or iodata\n- Conforms to the standard of `(en|de)code!?`, so it can be used with Plug\n- Conforms to the standard of accepting the last value set to a key in a Map\n- Conforms to 100% of the spec per [JSONTestSuite](https://github.com/nst/JSONTestSuite)\n- Extremely small and easy to embed\n\nWhy you should not use Tiny:\n\n- You want to decode JSON into Maps with atom keys\n- You want to decode JSON into module structs\n- You want to encode JSON into anything but unicode (although you could convert yourself)\n- You want hints as to why your JSON could not be parsed\n- You want the fastest JSON handling; there are many better options in this case.\n\nRealistically, most people will likely opt for one of the other parsers available. This project is largely here for the niche of bundling without Mix, and also just for example purposes.","ref":"readme.html#benefits"},{"type":"extras","title":"Installation - Tiny","doc":"Tiny is on Hex, and you can install it using the usual:\n\n  1. Add `tiny` to your list of dependencies in `mix.exs`:\n\n    ```elixir\n    def deps do\n      [{:tiny, \"~> 1.0\"}]\n    end\n    ```\n\n  2. Ensure `tiny` is started before your application:\n\n    ```elixir\n    def application do\n      [applications: [:tiny]]\n    end\n    ```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - Tiny","doc":"As mentioned, usage is intentionally super simple (this is the entire interface):\n\n```elixir\n# test input for decode\niex(1)> input = \"{\\\"key\\\":\\\"value\\\"}\"\n\"{\\\"key\\\":\\\"value\\\"}\"\n# \"safe\" decoding\niex(2)> { :ok, terms } = Tiny.decode(input)\n{:ok, %{\"key\" => \"value\"}}\n# \"unsafe\" decoding (raises ArgumentError)\niex(3)> terms = Tiny.decode!(input)\n%{\"key\" => \"value\"}\n# \"safe\" encoding\niex(4)> { :ok, json } = Tiny.encode(terms)\n{:ok, \"{\\\"key\\\":\\\"value\\\"}\"}\n# \"unsafe\" encoding (raises ArgumentError)\niex(5)> json = Tiny.encode!(terms)\n\"{\\\"key\\\":\\\"value\\\"}\"\n```\n\nIf your input is invalid, you'll receive an `ArgumentError`. If you're running your code in production, it's a waste of time to give you the character causing the error seeing you don't know the JSON you just tried to parse. If you do have it to hand (from your logs or something), you can just drop it in a JSON linter and it'll show you where the error is - there's no need to incur the overhead on every successful parse request. If you don't find this desirable, you should use Poison instead as it does keep track of the column/character where errors occur.\n\nAll of the functions shown above do accept a second optional argument containing a `Keyword` list of options. At the moment, the only option supported is `:iodata` in the `encode` functions, which will output your encoded JSON as `iodata` rather than a binary if truthy (please note \"truthy\" and not `true`). This flag is the same as the current (3.x) Poison option, for convenience.","ref":"readme.html#usage"},{"type":"extras","title":"Testing and Validation - Tiny","doc":"I have opted to use [JSONTestSuite](https://github.com/nst/JSONTestSuite) to test Tiny, as it is extremely extensive - particularly in terms of negative tests. Tiny passes every `y_*` and `n_*` test based on the official JSON specification.\n\nSeveral of the implementation specific tests (`i_*`) fail due to the nature of Elixir (invalid floats, codepoints, etc) - but don't worry, these tests are outside the official spec which means you're probably not dealing with JSON like that anyway. There are also several tests inside this repo to cover the basic use cases - they're not super thorough, but should cover everything enough to have a valid CI build.","ref":"readme.html#testing-and-validation"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}